@@ -1,7 +1,7 @@
 /* menu.c
 
 Copyright (C) 1999-2003 Tom Gilbert.
-Copyright (C) 2010-2011 Daniel Friesel.
+Copyright (C) 2010-2018 Daniel Friesel.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -49,14 +49,39 @@ void feh_menu_cb(feh_menu * m, feh_menu_item * i, int action, unsigned short dat
 void feh_menu_cb_opt_fullscreen(feh_menu * m, feh_menu_item * i);
 
 enum {
-	CB_CLOSE = 1, CB_EXIT, CB_RELOAD, CB_REMOVE, CB_DELETE, CB_RESET,
-	CB_REMOVE_THUMB, CB_DELETE_THUMB, CB_BG_TILED, CB_BG_SCALED,
-	CB_BG_CENTERED, CB_BG_FILLED, CB_BG_TILED_NOFILE,
-	CB_BG_SCALED_NOFILE, CB_BG_CENTERED_NOFILE, CB_BG_FILLED_NOFILE,
-	CB_SORT_FILENAME, CB_SORT_IMAGENAME, CB_SORT_FILESIZE, CB_SORT_RANDOMIZE,
-	CB_SAVE_IMAGE, CB_SAVE_FILELIST, CB_FIT, CB_OPT_DRAW_FILENAME,
-	CB_OPT_DRAW_ACTIONS, CB_OPT_KEEP_HTTP, CB_OPT_FREEZE_WINDOW,
-	CB_OPT_FULLSCREEN, CB_EDIT_ROTATE, CB_OPT_AUTO_ZOOM, CB_OPT_KEEP_ZOOM_VP
+	CB_CLOSE = 1,
+	CB_EXIT,
+	CB_RELOAD,
+	CB_REMOVE,
+	CB_DELETE,
+	CB_RESET,
+	CB_REMOVE_THUMB,
+	CB_DELETE_THUMB,
+	CB_BG_TILED,
+	CB_BG_SCALED,
+	CB_BG_CENTERED,
+	CB_BG_FILLED,
+	CB_BG_TILED_NOFILE,
+	CB_BG_SCALED_NOFILE,
+	CB_BG_CENTERED_NOFILE,
+	CB_BG_FILLED_NOFILE,
+	CB_SORT_FILENAME,
+	CB_SORT_IMAGENAME,
+	CB_SORT_DIRNAME,
+	CB_SORT_MTIME,
+	CB_SORT_FILESIZE,
+	CB_SORT_RANDOMIZE,
+	CB_SAVE_IMAGE,
+	CB_SAVE_FILELIST,
+	CB_FIT,
+	CB_OPT_DRAW_FILENAME,
+	CB_OPT_DRAW_ACTIONS,
+	CB_OPT_KEEP_HTTP,
+	CB_OPT_FREEZE_WINDOW,
+	CB_OPT_FULLSCREEN,
+	CB_EDIT_ROTATE,
+	CB_OPT_AUTO_ZOOM,
+	CB_OPT_KEEP_ZOOM_VP
 };
 
 feh_menu *feh_menu_new(void)
@@ -923,7 +948,9 @@ void feh_menu_init_common()
 
 	feh_menu_add_entry(m, "By File Name", NULL, CB_SORT_FILENAME, 0, NULL);
 	feh_menu_add_entry(m, "By Image Name", NULL, CB_SORT_IMAGENAME, 0, NULL);
-	if (opt.preload || (opt.sort > SORT_FILENAME))
+	feh_menu_add_entry(m, "By Directory Name", NULL, CB_SORT_DIRNAME, 0, NULL);
+	feh_menu_add_entry(m, "By Modification Date", NULL, CB_SORT_MTIME, 0, NULL);
+	if (opt.preload || (opt.sort > SORT_MTIME))
 		feh_menu_add_entry(m, "By File Size", NULL, CB_SORT_FILESIZE, 0, NULL);
 	feh_menu_add_entry(m, "Randomize", NULL, CB_SORT_RANDOMIZE, 0, NULL);
 
@@ -1218,7 +1245,7 @@ void feh_menu_cb(feh_menu * m, feh_menu_item * i, int action, unsigned short dat
 			if (m->fehwin->has_rotated) {
 				m->fehwin->im_w = gib_imlib_image_get_width(m->fehwin->im);
 				m->fehwin->im_h = gib_imlib_image_get_height(m->fehwin->im);
-				winwidget_resize(m->fehwin, m->fehwin->im_w, m->fehwin->im_h);
+				winwidget_resize(m->fehwin, m->fehwin->im_w, m->fehwin->im_h, 0);
 			}
 			winwidget_reset_image(m->fehwin);
 			winwidget_render_image(m->fehwin, 1, 0);
@@ -1252,6 +1279,18 @@ void feh_menu_cb(feh_menu * m, feh_menu_item * i, int action, unsigned short dat
 				slideshow_change_image(m->fehwin, SLIDE_FIRST, 1);
 			}
 			break;
+		case CB_SORT_DIRNAME:
+			filelist = gib_list_sort(filelist, feh_cmp_dirname);
+			if (opt.jump_on_resort) {
+				slideshow_change_image(m->fehwin, SLIDE_FIRST, 1);
+			}
+			break;
+		case CB_SORT_MTIME:
+			filelist = gib_list_sort(filelist, feh_cmp_mtime);
+			if (opt.jump_on_resort) {
+				slideshow_change_image(m->fehwin, SLIDE_FIRST, 1);
+			}
+			break;
 		case CB_SORT_FILESIZE:
 			filelist = gib_list_sort(filelist, feh_cmp_size);
 			if (opt.jump_on_resort) {
